use esp_idf_svc::hal::{
    prelude::*,
    gpio::AnyIOPin,
    spi::{
        config::{Config, DriverConfig},
        Dma,
        SpiBusDriver,
        SpiDriver,
    }
};

use ws2812_spi::Ws2812;
use ws2812_esp32_rmt_driver::lib_smart_leds::Ws2812Esp32Rmt;
use ws2812_esp32_rmt_driver::lib_embedded_graphics::{LedPixelShape, Ws2812DrawTarget};

use embedded_graphics::prelude::{Size, Point};

use crate::smart_leds_lib::SmartLedDisplay;
use crate::embedded_graphics_lib::EmbeddedDisplay;

use crate::render::{Surface, Display};
use crate::task::Task;

pub trait DisplayInit {
    fn new_display<S: Surface>() -> impl Display<S> + Task;
}

impl DisplayInit for Ws2812Esp32Rmt<'_> {
    fn new_display<S: Surface>() -> impl Display<S> + Task {
        let peripherals = Peripherals::take().unwrap();
        let led_pin = peripherals.pins.gpio14;
        let channel = peripherals.rmt.channel0;

        const POWER_VOLTS : u32 = 5;
        const POWER_MA : u32 = 500;
        const MAX_POWER_MW : u32 = POWER_VOLTS * POWER_MA;

        let target = Self::new(channel, led_pin).unwrap();
        return SmartLedDisplay::new(target, MAX_POWER_MW);
    }
}

impl<Shape: LedPixelShape> DisplayInit for Ws2812DrawTarget<'_, Shape> {
    fn new_display<S: Surface>() -> impl Display<S> + Task {
        let peripherals = Peripherals::take().unwrap();
        let led_pin = peripherals.pins.gpio14;
        let channel = peripherals.rmt.channel0;

        const POWER_VOLTS : u32 = 5;
        const POWER_MA : u32 = 500;
        const MAX_POWER_MW : u32 = POWER_VOLTS * POWER_MA;

        let target = Self::new(channel, led_pin).unwrap();
        return EmbeddedDisplay::<Self, S>::new(target, MAX_POWER_MW);
    }
}

pub struct SPIDisplay {}
impl DisplayInit for SPIDisplay {
    fn new_display<S: Surface>() -> impl Display<S> + Task {
        let peripherals = Peripherals::take().unwrap();

        let driver = SpiDriver::new_without_sclk(
            peripherals.spi2,
            peripherals.pins.gpio14,
            Option::<AnyIOPin>::None,
            &DriverConfig::new().dma(Dma::Auto(512))
        ).unwrap();

        let cfg = Config::new().baudrate(3_200.kHz().into());

        let spi = SpiBusDriver::new(driver, &cfg).unwrap();

        const POWER_VOLTS : u32 = 5;
        const POWER_MA : u32 = 500;
        const MAX_POWER_MW : u32 = POWER_VOLTS * POWER_MA;
        let target = Ws2812::new(spi);
        return SmartLedDisplay::new(target, MAX_POWER_MW)
    }
}


pub struct PonderjarMatrix {}
impl LedPixelShape for PonderjarMatrix {
    fn size() -> Size {
        Size::new(17, 17)
    }

    fn pixel_index(point: Point) -> Option<usize> {
        if (0..Self::size().width as i32).contains(&point.x) && (0..Self::size().height as i32).contains(&point.y) {
            if point.y % 2 == 0 {
                Some((point.y as u32 * Self::size().width as u32 + point.x as u32).try_into().unwrap())
            } else {
                Some((point.y as u32 * Self::size().width as u32 - point.x as u32).try_into().unwrap())
            }
        } else {
            None
        }
    }
}

pub type PonderjarTarget<'a> = Ws2812DrawTarget<'a, PonderjarMatrix>;
